==> unity-ads-4.10.0-sources.jar: com/unity3d/ads/core/domain/events/DiagnosticEventObserver.kt <==
package com.unity3d.ads.core.domain.events

import com.google.protobuf.kotlin.toByteString
import com.unity3d.ads.core.data.datasource.UniversalRequestDataSource
import com.unity3d.ads.core.data.repository.DiagnosticEventRepository
import com.unity3d.ads.core.domain.GetUniversalRequestForPayLoad
import com.unity3d.ads.core.domain.work.BackgroundWorker
import com.unity3d.ads.core.domain.work.DiagnosticEventJob
import com.unity3d.ads.core.domain.work.UniversalRequestWorkerData
import gateway.v1.UniversalRequestKt.payload
import kotlinx.coroutines.CoroutineDispatcher
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.getAndUpdate
import kotlinx.coroutines.flow.launchIn
import kotlinx.coroutines.flow.onEach
import kotlinx.coroutines.withContext
import java.util.UUID

class DiagnosticEventObserver(
    private val getUniversalRequestForPayLoad: GetUniversalRequestForPayLoad,
    private val getDiagnosticEventBatchRequest: GetDiagnosticEventBatchRequest,
    private val defaultDispatcher: CoroutineDispatcher,
    private val diagnosticEventRepository: DiagnosticEventRepository,
    private val universalRequestDataSource: UniversalRequestDataSource,
    private val backgroundWorker: BackgroundWorker
) {
    private val isRunning = MutableStateFlow(false)

    suspend operator fun invoke() = withContext(defaultDispatcher) {
        // If we are already collecting events, do nothing
        if (isRunning.getAndUpdate { true }) return@withContext

        diagnosticEventRepository.diagnosticEvents.onEach { currentDiagnosticEventRequest ->
            val payload = payload {
                diagnosticEventRequest = getDiagnosticEventBatchRequest(currentDiagnosticEventRequest)
            }
            val fullRequest = getUniversalRequestForPayLoad(payload)

            val workId = UUID.randomUUID().toString()
            universalRequestDataSource.set(workId, fullRequest.toByteArray().toByteString())

            val universalRequestWorkerData = UniversalRequestWorkerData(workId)
            backgroundWorker<DiagnosticEventJob>(universalRequestWorkerData)

        }.launchIn(CoroutineScope(defaultDispatcher))
    }
}

==> unity-ads-4.11.1-sources.jar: com/unity3d/ads/core/domain/events/DiagnosticEventObserver.kt <==
package com.unity3d.ads.core.domain.events

import com.google.protobuf.kotlin.toByteString
import com.unity3d.ads.core.data.datasource.UniversalRequestDataSource
import com.unity3d.ads.core.data.repository.DiagnosticEventRepository
import com.unity3d.ads.core.domain.GetUniversalRequestForPayLoad
import com.unity3d.ads.core.domain.work.BackgroundWorker
import com.unity3d.ads.core.domain.work.DiagnosticEventJob
import com.unity3d.ads.core.domain.work.UniversalRequestWorkerData
import com.unity3d.services.core.di.ServiceProvider.DEFAULT_DISPATCHER
import gateway.v1.UniversalRequestKt.payload
import kotlinx.coroutines.CoroutineDispatcher
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.getAndUpdate
import kotlinx.coroutines.flow.launchIn
import kotlinx.coroutines.flow.onEach
import kotlinx.coroutines.withContext
import org.koin.core.annotation.Named
import org.koin.core.annotation.Single
import java.util.UUID

@Single
class DiagnosticEventObserver(
    private val getUniversalRequestForPayLoad: GetUniversalRequestForPayLoad,
    private val getDiagnosticEventBatchRequest: GetDiagnosticEventBatchRequest,
    @Named(DEFAULT_DISPATCHER) private val defaultDispatcher: CoroutineDispatcher,
    private val diagnosticEventRepository: DiagnosticEventRepository,
    private val universalRequestDataSource: UniversalRequestDataSource,
    private val backgroundWorker: BackgroundWorker
) {
    private val isRunning = MutableStateFlow(false)

    suspend operator fun invoke() = withContext(defaultDispatcher) {
        // If we are already collecting events, do nothing
        if (isRunning.getAndUpdate { true }) return@withContext

        diagnosticEventRepository.diagnosticEvents.onEach { currentDiagnosticEventRequest ->
            val payload = payload {
                diagnosticEventRequest = getDiagnosticEventBatchRequest(currentDiagnosticEventRequest)
            }
            val fullRequest = getUniversalRequestForPayLoad(payload)

            val workId = UUID.randomUUID().toString()
            universalRequestDataSource.set(workId, fullRequest.toByteArray().toByteString())

            val universalRequestWorkerData = UniversalRequestWorkerData(workId)
            backgroundWorker<DiagnosticEventJob>(universalRequestWorkerData)

        }.launchIn(CoroutineScope(defaultDispatcher))
    }
}
